﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Web.Http;

namespace HSolutions.Framework.MicroServices.Controllers
{
    public class BaseController : ApiController
    {
        /// <summary>
        /// Método para teste do Serviço
        /// </summary>
        /// <param></param>
        /// <returns></returns>
        public string GetBaseTeste()
        {
            return String.Concat("[", DateTime.Now, "] ", Assembly.GetExecutingAssembly().FullName);
        }

        /// <summary>
        /// API - Lista todos os métodos do serviço
        /// </summary>
        /// <param></param>
        /// <returns></returns>
        public dynamic GetBaseAPI()
        {
            var api = new List<APIClass>();

            var methods = (this.GetType()).GetMethods(BindingFlags.Public | BindingFlags.Instance | BindingFlags.DeclaredOnly);
            foreach (var met in methods)
            {
                api.Add(new APIClass { Name = met.Name, Return = met.ReturnType.ToString(), Params = FormatarParametros(met.GetParameters()).ToList() });
            }

            return api;
        }
        
        internal IEnumerable<string> FormatarParametros(ParameterInfo[] parameters)
        {
            foreach (var param in parameters)
                yield return param.Name + " (" + param.ParameterType + ")";
        }
    }

    /// <summary>
    /// Classe consumida pelo método de API
    /// </summary>
    public class APIClass
    {
        public string Name { get; set; }
        public IList<string> Params { get; set; }
        public string Return { get; set; }
    }
}
